use reqwest::{blocking::Client, blocking::ClientBuilder, header::HeaderValue, redirect::Policy};
use scraper::{Html, Selector};

const URL: &str = "https://rentry.co/Hyprland-controversy/edit";

enum DeleteResult {
    Success,
    InvalidEditCode,
    UnknownFailure(String),
    RequestFailure(String),
}

#[derive(Debug)]
struct CSRFData {
    cookie: String,
    form: String,
}

fn get_cookie_value(header_value: &HeaderValue, cookie_name: &str) -> Option<String> {
    let cookies_str = header_value.to_str().ok()?;
    cookies_str.split(';').find_map(|cookie| {
        let mut parts = cookie.split('=');
        let name = parts.next()?.trim();
        let value = parts.next()?.trim();
        if name == cookie_name {
            Some(value.to_string())
        } else {
            None
        }
    })
}

fn get_csrf_data(client: &Client) -> CSRFData {
    let res = client.get(URL).send().unwrap();
    let cookies = res.headers().get("Set-Cookie").unwrap();
    let cookie_middleware_token = get_cookie_value(cookies, "csrftoken").unwrap();

    let body = res.text().unwrap();
    let document = Html::parse_document(&body);
    let selector = Selector::parse("input[name=\"csrfmiddlewaretoken\"]").unwrap();
    let form_middleware_token = document
        .select(&selector)
        .next()
        .unwrap()
        .value()
        .attr("value")
        .unwrap();

    CSRFData {
        cookie: cookie_middleware_token,
        form: form_middleware_token.to_string(),
    }
}

fn attempt_delete(client: &Client, csrf_data: &CSRFData, edit_code: &str) -> DeleteResult {
    let res = client
        .post(URL)
        .header("Referer", URL)
        .header(
            "Cookie",
            "csrftoken=".to_string() + csrf_data.cookie.as_str(),
        )
        .form(&[
            ("csrfmiddlewaretoken", csrf_data.form.as_str()),
            ("edit_code", edit_code),
            ("new_edit_code", ""),
            ("new_url", ""),
            ("text", ""),
            ("delete", "delete"),
        ])
        .send()
        .unwrap();

    // Get these before obtaining text, as that consumes the response
    let status = res.status();
    let headers = res.headers().clone();

    let raw_txt = res.text().unwrap();
    let document = Html::parse_document(&raw_txt);

    if status.is_redirection() {
        let location = headers.get("Location").unwrap().to_str().unwrap();
        if location == "/" {
            return DeleteResult::Success;
        }
        return DeleteResult::RequestFailure("unexpected redirection: ".to_string() + location);
    }

    if status.is_success() {
        let selector =
            Selector::parse("fieldset > div.text-danger.messages > ul.errorlist > li").unwrap();
        if let Some(element) = document.select(&selector).next() {
            let txt = element.text().collect::<String>();
            if txt == "Invalid edit code." {
                return DeleteResult::InvalidEditCode;
            }
            return DeleteResult::UnknownFailure("form error: ".to_string() + &txt);
        }

        return DeleteResult::UnknownFailure(format!("unknown failure: {raw_txt}"));
    }

    let selector = Selector::parse("span").unwrap();
    let err_txt = document
        .select(&selector)
        .next()
        .unwrap()
        .text()
        .collect::<String>();
    let err_txt = err_txt.trim().to_string();

    DeleteResult::RequestFailure(err_txt)
}

fn main() {
    let client = ClientBuilder::new()
        .redirect(Policy::none())
        .build()
        .unwrap();
    let csrf_data = get_csrf_data(&client);
    println!("Token: {csrf_data:?}");

    match attempt_delete(&client, &csrf_data, "qwack") {
        DeleteResult::Success => println!("Successfully deleted!"),
        DeleteResult::InvalidEditCode => println!("Failed to delete: Invalid edit code!"),
        DeleteResult::RequestFailure(txt) => println!("Failed to delete (req): {txt}!"),
        DeleteResult::UnknownFailure(txt) => println!("Failed to delete (unknown): {txt}!"),
    }
}
